package api

import (
	"cockData/server/pkg"
	"cockData/server/sql"
	"github.com/gin-gonic/gin"
)

type DiamondParams struct {
	Diamond int64  `binding:"required"`
	Reason  string `binding:"required"`
}

type DiamondResults struct {
	CurDiamond int64  `json:"cur_diamond"`
	CreateTime string `json:"create_time"`
}

func changeDiamond(context *gin.Context) {
	var params DiamondParams
	userID, err := CheckParamsAndOpenId(context, &params)
	if err != nil {
		return
	}
	curDiamond, err := sql.ChangeDiamond(userID, params.Diamond, params.Reason)
	ResponseApiWithError(context, CreateSuccessResponse(DiamondResults{CurDiamond: curDiamond}), err)
}

func getDiamond(context *gin.Context) {
	userID, err := GetOpenId(context)
	if err != nil {
		return
	}
	createAt, curDiamond, err := sql.GetUserInfo(userID)
	ResponseApiWithError(context, CreateSuccessResponse(DiamondResults{CreateTime: pkg.GetFormatTime(createAt), CurDiamond: curDiamond}), err)
}
